;    Filename:                                                                 *
;    Date:                                                                     *
;    File Version:                                                             *
;    Author:                                                                   *
;    Company:                                                                  *
;    Description:                                                              *

    #include "p16f628a.inc"

; CONFIG
; __config 0xFF5C
 __CONFIG _FOSC_INTOSCIO & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _BOREN_OFF & _LVP_OFF & _CPD_OFF & _CP_OFF

 
;*******************************************************************************
;
; Refer to datasheet for available data memory (RAM) organization assuming
; relocatible code organization (which is an option in project
; properties > mpasm (Global Options)).  Absolute mode generally should
; be used sparingly.
;
; Example of using GPR Uninitialized Data
;
;   GPR_VAR        UDATA
;   MYVAR1         RES        1      ; User variable linker places
;   MYVAR2         RES        1      ; User variable linker places
;   MYVAR3         RES        1      ; User variable linker places
;
;   ; Example of using Access Uninitialized Data Section (when available)
;   ; The variables for the context saving in the device datasheet may need
;   ; memory reserved here.
;   INT_VAR        UDATA_ACS
;   W_TEMP         RES        1      ; w register for context saving (ACCESS)
;   STATUS_TEMP    RES        1      ; status used for context saving
;   BSR_TEMP       RES        1      ; bank select used for ISR context saving
;
;*******************************************************************************

count1	    equ	    0x20	    ;used in delay routine
counta	    equ	    0x21	    ;used in delay routine
countb	    equ	    0x22
count	    equ	    0x23
pwondelay   equ	    0x24

key	    equ	    0x30	    ;select register for input selection
input	    equ	    0x31	    ;bits with the selected inputs
mode	    equ	    0x33	    ;panic mode, mute etc
	    
keyout	    equ	    PORTA
keyin	    equ	    PORTB
	    
portastat   equ	    0x40
	    
;constants
Tape2PB	    equ	    0
Tape2Rec    equ	    1
Tape1PB	    equ	    2
Tape1Rec    equ	    3
TunerIn	    equ	    4
CDIn	    equ	    5
AuxIn	    equ	    6
PhonoIn	    equ	    7
	    
modpanic    equ	    0
modmute	    equ	    1
		    
;*******************************************************************************
; Reset Vector
;*******************************************************************************

;RES_VECT
    org	    0x000		    ; processor reset vector
    goto    Start                   ; go to beginning of program
    org	    0x005

; TODO INSERT ISR HERE

;*******************************************************************************
; MAIN PROGRAM
;*******************************************************************************

;MAIN_PROG code			    ; let linker place main program
Keys:
    addwf   PCL, f
    goto    Tape2
    goto    Tuner
    goto    CD
    goto    Tape1
    goto    Aux
    goto    Phono

Start:
    clrf    PORTA
    clrf    PORTB
    clrf    key
    clrf    mode
    clrf    input
    movlw   0x07
    movwf   CMCON
    bsf	    STATUS, RP0
    movlw   b'00100000'	;set PortA all outputs except RA5 (which is the alarm input)
    movwf   TRISA
    movlw   b'00000111' ;set portB 3x inputs (RB0 RB1 RB2 and rest is output)
    movwf   TRISB
    bcf	    STATUS, RP0
    
    call    Delay200
    
    bsf	    PORTA, RA4
    
;startup sequence
    ;wait 7 secs
    movlw   30
    movwf   pwondelay
    
power_on_delay:
    call    Delay100
    bcf	    PORTA, RA4	;LED on
    call    Delay100
    bsf	    PORTA, RA4	;LED on    
    decfsz  pwondelay, f
    goto    power_on_delay
    bcf	    PORTA, RA4	;LED on
    call    Delay200
    btfsc   PORTA, RA5  ;This is the input pin to check!
    goto    Protection
    bsf	    PORTB, RB3	;speaker relay ON
    goto    CD
    
    ;call    Delay200
    ;bcf	    PORTB,  RB5	;CD
    
;Keyscan  matrix
;***************
;scanning and select

Read:
    call    Scan
    movlw   0x06
    subwf   key, w
    btfss   STATUS, Z
    goto    KeyPressed	    ;if a button is pressed then go to keypressed
    goto    Read

Scan:
    clrf    key

Scan1:	    ;row 1 to 3
    bsf	    PORTA, RA2
    btfsc   PORTB, RB0	    ;Tape 2
    goto    Check
    incf    key, f
    btfsc   PORTB, RB1	    ;Tuner
    goto    Check
    incf    key, f
    btfsc   PORTB, RB2	    ;CD
    goto    Check
    incf    key, f
    bcf	    PORTA, RA2
    
    ;check protection
    btfsc   PORTA, RA5	    ;check if all is good
    goto    Protection
    
Scan2:	    ;row 4 to 6
    bsf	    PORTA, RA3
    btfsc   PORTB, RB0	    ;Tape 1
    goto    Check
    incf    key, f
    btfsc   PORTB, RB1	    ;Aux
    goto    Check
    incf    key, f
    btfsc   PORTB, RB2	    ;Phono
    goto    Check
    incf    key, f
    bcf	    PORTA, RA3
    
    ;check protection
    btfsc   PORTA, RA5	    ;check again if all is good
    goto    Protection 
    
Check:
    bcf	    PORTA, RA2	    
    bcf	    PORTA, RA3
    return
    
KeyPressed:
    movf    key, w
    goto    Keys

;Button preprocessing - selecting correct inputs
;***********************************************
Tape2:
    btfss   input, Tape2PB
    goto    Tape2On
    call    Mute
    goto    Read
Tape2On:
    btfsc   input, Tape1PB
    goto    Tape1to2
    bsf	    input, Tape2PB
    goto    Switch
Tape1to2:
    clrf    input
    bsf	    input, Tape2PB
    bsf	    input, Tape1Rec
    goto    Switch
   

Tape1:
    btfss   input, Tape1PB
    goto    Tape1On
    call    Mute
    goto    Read
Tape1On:
    btfsc   input, Tape2PB
    goto    Tape2to1
    bsf	    input, Tape1PB
    goto    Switch
Tape2to1:
    clrf    input
    bsf	    input, Tape1PB
    bsf	    input, Tape2Rec
    goto    Switch
    
Tuner:
    btfss   input, TunerIn
    goto    TunerOn
	btfsc   input, Tape1PB
	goto    TunerOn
	btfsc   input, Tape2PB
	goto    TunerOn
    call    Mute    ;goto MuteCircuit
    goto    Read
TunerOn:
    clrf    input
    bsf	    input, TunerIn
    goto    Switch


Aux:
    btfss   input, AuxIn
    goto    AuxOn
    	btfsc   input, Tape1PB
	goto    AuxOn
	btfsc   input, Tape2PB
	goto    AuxOn
    call    Mute
    goto    Read
AuxOn:
    clrf    input
    bsf	    input, AuxIn
    goto    Switch

    
CD:
    btfss   input, CDIn
    goto    CDOn
    	btfsc   input, Tape1PB
	goto    CDOn
	btfsc   input, Tape2PB
	goto    CDOn
    call    Mute
    goto    Read
CDOn:
    clrf    input
    bsf	    input, CDIn
    goto    Switch

    
Phono:
    btfss   input, PhonoIn
    goto    PhonoOn
    	btfsc   input, Tape1PB
	goto    PhonoOn
	btfsc   input, Tape2PB
	goto    PhonoOn
    call    Mute
    goto    Read
PhonoOn:
    clrf    input
    bsf	    input, PhonoIn
    goto    Switch

;Selection mode
;**************
Switch:
    ; this piece of code does the actual switching of relays
    bcf     PORTB, RB3
    btfss   mode, modmute 
    call    Delay200
    btfss   mode, modmute
    call    Delay200
    
    bcf	    PORTA, RA0	;tape2
    bcf	    PORTA, RA1	;tape2 rec
    bcf	    PORTA, RA6	;tape1
    bcf	    PORTA, RA7	;tape1 rec
    bcf	    PORTB, RB7  ;tuner
    bcf	    PORTB, RB6	;aux
    bcf	    PORTB, RB5	;cd
    bcf	    PORTB, RB4	;phono
    
    call    Delay200
    call    Delay200
        
    btfsc   input, Tape2PB
    bsf	    PORTA, RA0
    btfsc   input, Tape2Rec
    bsf	    PORTA, RA1
    btfsc   input, Tape1PB
    bsf	    PORTA, RA6
    btfsc   input, Tape1Rec
    bsf	    PORTA, RA7
    btfsc   input, TunerIn
    bsf	    PORTB, RB7
    btfsc   input, AuxIn
    bsf	    PORTB, RB6
    btfsc   input, CDIn
    bsf	    PORTB, RB5
    btfsc   input, PhonoIn
    bsf	    PORTB, RB4
    
    call    Delay200
    call    Delay200  
    
    ;And again... check if anything is wrong before turning on speakers
    ;Check protection before turning on speakers
    btfsc   PORTA, RA5
    goto    Protection 
    
    btfss   mode, modmute
    bsf     PORTB, RB3
    btfss   mode, modmute
    bcf	    PORTA, RA4	    ;mode back to normal    
    
    ;bcf	    mode, modmute   ;mode back to normal
    goto    Read
    
Mute:
    ;Code for activating / deactivationg mute mode. When one of the input
    ;selection buttons is pressed (if already switched on), mute is activated
    ;or deactivated when pressed one more time
    btfss   mode, modmute
    goto    MuteOn
    goto    MuteOff
MuteOff:
    ;Check protection before turning on speakers
    btfsc   PORTA, RA5
    goto    Protection
    bsf     PORTB, RB3
    bcf	    PORTA, RA4
    bcf	    mode, modmute
    call    Delay200
    return
MuteOn:   
    bcf	    PORTB, RB3
    bsf	    PORTA, RA4
    bsf	    mode, modmute
    call    Delay200
    return
    
Protection: ;infinite loop for full protection
    ;this routine is called when DC voltage is high or AC drops out. 
    ;(AC drops when amp is switched off)
    ;see circuit diagram for hardware. 
    call    Delay5
    btfss   PORTA, RA5
    goto    Read   
    clrf    PORTB
    clrf    PORTA
    bsf	    mode, modpanic
ProtectionLoop:
    bcf	    PORTA, RA4
    call    Delay200
    bsf	    PORTA, RA4
    call    Delay200   
    goto    ProtectionLoop
    
;********
;Just a couple of delay routines
;********
    
Delay200:
    movlw   d'200'	;delay 200ms
    goto    d0    
Delay100:
    movlw   d'100'	;delay 100mS
    goto    d0
Delay96:
    movlw   d'96'
    goto    d0
Delay48:
    movlw   d'48'	;delay 48mS
    goto    d0
Delay2:
    movlw   d'2'	;delay 2mS
    goto    d0
Delay1:
    movlw   d'1'	;delay 1mS
    goto    d0
Delay20:
    movlw   d'20'
    goto    d0
Delay4:
    movlw   d'4'
    goto    d0
Delay5:
    movlw   d'5'
d0:
    movwf   count1
d1:
    movlw   0xc7
    movwf   counta
    movlw   0x01
    movwf   countb
    
delsub:
    decfsz  counta, f
    goto    $+2
    decfsz  countb, f
    goto    delsub
    decfsz  count1, f
    goto    d1
    retlw   0x00
    
    end

